/*
 * Copyright (c) 2007, Markus Dolze <bsdfan@nurfuerspam.de>.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * A program which gathers statistics about usage of local network interface
 * and creates HTML graphs.
 */
#include "config.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/sysctl.h>
#include <sys/time.h>
#include <sys/param.h>

#include <net/if.h>
#include <net/if_mib.h>

#include <err.h>
#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <time.h>
#include <unistd.h>

#include "iograph.h"

#define INTEXT "<font color=\"#00CC00\">In:</font>"
#define OUTTEXT "<font color=\"#0000FF\">Out:</font>"

const char     *monthnames[] = {"January", "February", "March", "April", "May", "June", "July", "August",
"September", "October", "November", "December"};

struct db      *database;
char		ifname    [IF_NAMESIZE] = "ath0";
char		outdir    [MAXPATHLEN] = ".";
char		hostname  [_POSIX_HOST_NAME_MAX + 1] = "";

int
main(int argc, char *argv[])
{
	time_t		now, stat_t;
	struct tm      *tm_now, *tm_stat;
	int		datasize;
	int		ch;
	int		fd;
	char		datafile  [MAXPATHLEN] = "data.bin";

	/* Process command line options */
	while ((ch = getopt(argc, argv, "hi:d:o:")) != -1) {
		switch (ch) {
		case 'i':
			snprintf(ifname, sizeof(ifname), "%s", optarg);
			break;
		case 'd':
			snprintf(datafile, MAXPATHLEN, "%s", optarg);
			break;
		case 'o':
			snprintf(outdir, MAXPATHLEN, "%s", optarg);
			break;
		case 'h':
			/* FALLTHROUGH */
		case '?':
			/* FALLTHROUGH */
		default:
			printf("%s %s\n", PACKAGE, VERSION);
			printf("Usage: iograph [-i interface] [-d datafile] [-o outputdir]\n");
			exit(EX_USAGE);
		}
	}

	/* Read local time */
	now = time(NULL);
	if ((tm_now = calloc(1, sizeof(struct tm))) == NULL)
		err(EX_SOFTWARE, "Time buffer 1");
	localtime_r(&now, tm_now);
	printf("Local time: year=%i, month=%i, day=%i, hour=%i, minute=%i\n", 1900 + tm_now->tm_year, tm_now->tm_mon, tm_now->tm_mday,
	       tm_now->tm_hour, tm_now->tm_min);

	/*
	 * Create a second struct tm from current time substract 1 hour and
	 * recalculate the struct tm.
	 */
	if ((tm_stat = calloc(1, sizeof(struct tm))) == NULL)
		err(EX_SOFTWARE, "Time buffer 1");
	localtime_r(&now, tm_stat);
	tm_stat->tm_hour -= 1;
	stat_t = mktime(tm_stat);
	localtime_r(&stat_t, tm_stat);
	printf("Stat time: year=%i, month=%i, day=%i, hour=%i, minute=%i\n", 1900 + tm_stat->tm_year,
	       tm_stat->tm_mon, tm_stat->tm_mday, tm_stat->tm_hour, tm_stat->tm_min);

	/* Try to read the local hostname */
	if (gethostname(hostname, sizeof(hostname)) == -1)
		err(EX_SOFTWARE, "Failed to get host name");

	/* Alloc space for in memory database */
	if ((database = calloc(1, sizeof(struct db))) == NULL)
		err(EX_SOFTWARE, "Failed to alloc database");

	/* Open or create data file */
	if ((fd = open(datafile, O_RDWR | O_CREAT, 0640)) == -1)
		err(EX_CANTCREAT, "Cannot open data file");
	if ((datasize = read(fd, database, sizeof(struct db))) == -1) {
		close(fd);
		err(EX_IOERR, "Error reading datafile");
	} else if (datasize == 0) {
		/* do nothing, update data and exit */
	} else if (datasize != sizeof(struct db)) {
		errx(EX_DATAERR, "Size of data file does not match database size!");
	}

	/* Allways update transfer statistics and graph for the passed hour */
	updatedata(tm_stat->tm_hour);
	hourlyhtml(tm_stat);

	/*
	 * If the day has changed,
	 * - update statistics for the past day,
	 * - create a new hourly graph for the new day (the last for the past day
	 *   has been created above),
	 * - create the daily graph for the past day.
	 */
	if (tm_stat->tm_mday != tm_now->tm_mday) {
		newday(tm_stat->tm_mday);
		hourlyhtml(tm_now);
		dailyhtml(tm_stat, (tm_stat->tm_mon != tm_now->tm_mon));
	}

	/*
	 * If the month has changed,
	 * - update statistics for the past month.
	 * - create a new monthly graph.
	 * Note that even if the month changed, the daily graph is created
	 * by the part above.
	 */
	if (tm_stat->tm_mon != tm_now->tm_mon) {
		newmonth(tm_stat->tm_mon);
		monthlyhtml();
	}

	/* Update data file */
	lseek(fd, 0, SEEK_SET);
	write(fd, database, sizeof(struct db));
	close(fd);

	return EX_OK;
}


/*
 * Requests statistics and takes care of interface resets and counter wraps.
 * Updates the hourly database values.
 */
void
updatedata(int lasthour)
{
	bytecount	current;
	time_t		lastIfChange;
	/* Read in interface statistics and last interface change date */
	if (get_iface_stats(&current, &lastIfChange) == -1) {
		warnx("Error reading interface statistics!\n");
		return;
	}
	/* if the interface has changes, counters are reset */
	if ((database->ifchange != lastIfChange)) {
		database->last.rx = database->last.tx = 0;
		database->ifchange = lastIfChange;
	}
	/*
	 * Calculate difference to previous run, taking possible integer
	 * overruns into account.
	 */
	if (current.rx < database->last.rx)
		database->hours[lasthour].rx = (ULONG_MAX - database->last.rx) + current.rx;
	else
		database->hours[lasthour].rx = current.rx - database->last.rx;

	if (current.tx < database->last.tx)
		database->hours[lasthour].tx = (ULONG_MAX - database->last.tx) + current.tx;
	else
		database->hours[lasthour].tx = current.tx - database->last.tx;

	/* Save current values */
	database->last.rx = current.rx;
	database->last.tx = current.tx;

	printf("Interface stats for hour %i: %llu bytes in, %llu bytes out\n", lasthour, database->hours[lasthour].rx,
	       database->hours[lasthour].tx);
}


/*
 * Returns the number of days in a given month. The value for February takes
 * leap years into account.
 */
int
last_mday(int month, int year)
{
	int mdays[12] = {31, 0, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

	return (month == 1) ? 28 + ((year % 100 && !(year % 4)) || !(year % 400)) : mdays[month];
}


/*
 * Aggregates statistics for the last day from the hourly values. Hourly
 * values are cleared and the hourly file saved under in the format
 * dayXX.html, where XX is the number of day in month.
 */
void
newday(int lastday)
{
	int		a;
	bytecount	total;
	char		oldfile   [MAXPATHLEN], newfile[MAXPATHLEN];

	/* Calculate and store totals */
	total.rx = total.tx = 0;
	for (a = 0; a <= 23; a++) {
		total.rx += database->hours[a].rx;
		total.tx += database->hours[a].tx;
	}
	/* Shift lastday by one, because day is counted from 1 on. */
	database->days[lastday - 1] = total;

	/* Clear previous data */
	for (a = 0; a <= 23; a++) {
		database->hours[a].rx = database->hours[a].tx = 0;
	}

	/* Move old hourly HTML file to one with the name of day */
	snprintf(oldfile, MAXPATHLEN, "%s/hourly.html", outdir);
	snprintf(newfile, MAXPATHLEN, "%s/day%i.html", outdir, lastday);
	if (rename(oldfile, newfile) == -1)
		warn("Cannot move hourly file");
}


/*
 * Aggregates statistics for the last month from the daily values. Daily
 * values are cleared and the daily file saved under in the format
 * monthXX.html, where XX is the number of the month.
 */
void
newmonth(int lastmonth)
{
	int		a;
	bytecount	total;
	char		oldfile   [MAXPATHLEN], newfile[MAXPATHLEN];

	total.rx = total.tx = 0;
	for (a = 0; a < 31; a++) {
		total.rx += database->days[a].rx;
		total.tx += database->days[a].tx;
	}

	database->months[lastmonth] = total;

	for (a = 0; a < 31; a++) {
		database->days[a].rx = database->days[a].tx = 0;
	}

	snprintf(oldfile, MAXPATHLEN, "%s/daily.html", outdir);
	snprintf(newfile, MAXPATHLEN, "%s/month%i.html", outdir, lastmonth + 1);
	if (rename(oldfile, newfile) == -1)
		warn("Cannot move daily file");
}


/*
 * Creates the HTML file with the statistics for the current day.
 */
void
hourlyhtml(struct tm *date)
{
	FILE           *html_fd;
	int		a;
	unsigned long long besthour;
	float		hscale;
	bytecount	total;
	char		pname     [MAXPATHLEN];

	printf("Creating hourly HTML for: %s", asctime(date));

	snprintf(pname, MAXPATHLEN, "%s/hourly.html", outdir);
	if ((html_fd = fopen(pname, "w")) == NULL)
		err(EX_CANTCREAT, "Cannot create hourly html file");

	/*
	 * Initializing to 1 does not distort results but prevents division
	 * by zero
	 */
	besthour = 1;
	for (a = 0; a <= 23; a++) {
		if (database->hours[a].rx > besthour)
			besthour = database->hours[a].rx;
		if (database->hours[a].tx > besthour)
			besthour = database->hours[a].tx;
	}
	hscale = (float)besthour / 450;

	fprintf(html_fd, "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n\"http://www.w3.org/TR/html4/loose.dtd\">\n");
	fprintf(html_fd, "<html>\n<head><title>"PACKAGE" - "VERSION"</title><meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\"></head>\n");
	fprintf(html_fd, "<body bgcolor=\"#FFFFFF\">\n");

	fprintf(html_fd, "<p>Today | ");
	snprintf(pname, MAXPATHLEN, "%s/daily.html", outdir);
	if (doesfileexist(pname))
		fprintf(html_fd, "<a href=\"daily.html\">This month</a> | ");
	else
		fprintf(html_fd, "This month | ");
	snprintf(pname, MAXPATHLEN, "%s/months.html", outdir);
	if (doesfileexist(pname))
		fprintf(html_fd, "<a href=\"months.html\">Previous Months</a></p>\n");
	else
		fprintf(html_fd, "Previous Months</p>\n");

	fprintf(html_fd, "<h1 style=\"font-size:1.3em\">%s %s - Network I/O for %s</h1>\n", monthnames[date->tm_mon], daystr(date->tm_mday), hostname);
	fprintf(html_fd, "<table><tr><td><table cellpadding=\"5\" border=\"1\">\n");
	total.rx = total.tx = 0;
	for (a = 0; a <= 23; a++) {
		total.rx += database->hours[a].rx;
		total.tx += database->hours[a].tx;
		fprintf(html_fd, "<tr><td><b>%02i:00</b></td>", a);
		fprintf(html_fd, "<td width=\"455\"><img src=\"in.gif\" height=\"12\" width=\"%.f\" alt=\"ingraph\"><br>",
			database->hours[a].rx / hscale);
		fprintf(html_fd, "<img src=\"out.gif\" height=\"12\" width=\"%.f\" alt=\"outgraph\"></td>",
			database->hours[a].tx / hscale);
		fprintf(html_fd, "<td>" INTEXT " %s<br>" OUTTEXT " %s</td></tr>\n", sizestr(database->hours[a].rx),
			sizestr(database->hours[a].tx));
	}
	fprintf(html_fd, "</table></td></tr>\n");

	fprintf(html_fd, "<tr><td style=\"text-align: right\">Today's total: %s<br>" INTEXT " %s - " OUTTEXT "%s</td></tr>\n</table>\n",
	sizestr(total.rx + total.tx), sizestr(total.rx), sizestr(total.tx));

	fprintf(html_fd, "<p><img src=\"iograph.gif\" alt=\"logo\" style=\"float:left; margin-right: 1em\">"PACKAGE" - "VERSION"<br><a href=\"http://mdolze.gmxhome.de/\">mdolze.gmxhome.de</a></p>\n");
	fprintf(html_fd, "</body>\n</html>");
	fclose(html_fd);
}


/*
 * Creates the HTML file with the statistics for the current month.
 */
void
dailyhtml(struct tm *date, int isNewMonth)
{
	FILE           *html_fd;
	int		a         , endday;
	unsigned long long bestday;
	float		hscale;
	bytecount	total;
	char		pname     [MAXPATHLEN];

	printf("Creating daily HTML for: %s", asctime(date));

	snprintf(pname, MAXPATHLEN, "%s/daily.html", outdir);
	if ((html_fd = fopen(pname, "w")) == NULL)
		err(EX_CANTCREAT, "Cannot create daily html file");

	endday = last_mday(date->tm_mon, date->tm_year);

	/*
	 * Initializing to 1 does not distort results but prevents division
	 * by zero
	 */
	bestday = 1;
	for (a = 0; a < endday; a++) {
		if (database->days[a].rx > bestday)
			bestday = database->days[a].rx;
		if (database->days[a].tx > bestday)
			bestday = database->days[a].tx;
	}
	hscale = (float)bestday / 450;

	fprintf(html_fd, "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n\"http://www.w3.org/TR/html4/loose.dtd\">\n");
	fprintf(html_fd, "<html>\n<head><title>"PACKAGE" - "VERSION"</title><meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\"></head>\n");
	fprintf(html_fd, "<body bgcolor=\"#FFFFFF\">\n");

	snprintf(pname, MAXPATHLEN, "%s/hourly.html", outdir);
	if (doesfileexist(pname))
		fprintf(html_fd, "<p><a href=\"hourly.html\">Today</a> | ");
	else
		fprintf(html_fd, "<p>Today | ");
	fprintf(html_fd, "This month | ");
	snprintf(pname, MAXPATHLEN, "%s/months.html", outdir);
	if (!isNewMonth && doesfileexist(pname))
		fprintf(html_fd, "<a href=\"months.html\">Previous Months</a></p>\n");
	else
		fprintf(html_fd, "Previous Months</p>\n");

	fprintf(html_fd, "<h1 style=\"font-size:1.3em\">%s - Network I/O for %s</h1>\n", monthnames[date->tm_mon], hostname);
	fprintf(html_fd, "<table><tr><td><table cellpadding=\"5\" border=\"1\">\n");
	total.rx = total.tx = 0;
	for (a = 0; a < endday; a++) {
		total.rx += database->days[a].rx;
		total.tx += database->days[a].tx;

		snprintf(pname, MAXPATHLEN, "%s/day%i.html", outdir, a + 1);
		/*
		 * Only link to hourly graph if that file 
		 * exists and for the past days.
		 */
		if (doesfileexist(pname) && a <= (date->tm_mday - 1))
			fprintf(html_fd, "<tr><td><a href=\"day%i.html\"><b>%s</b></a></td>", a + 1, daystr(a + 1));
		else
			fprintf(html_fd, "<tr><td><b>%s</b></td>", daystr(a + 1));

		fprintf(html_fd, "<td width=\"455\"><img src=\"in.gif\" height=\"12\" width=\"%.f\" alt=\"ingraph\"><br>",
			database->days[a].rx / hscale);
		fprintf(html_fd, "<img src=\"out.gif\" height=\"12\" width=\"%.f\" alt=\"outgraph\"></td>", 
			database->days[a].tx / hscale);
		fprintf(html_fd, "<td>" INTEXT " %s<br>" OUTTEXT " %s</td></tr>\n", sizestr(database->days[a].rx),
			sizestr(database->days[a].tx));
	}
	fprintf(html_fd, "</table></td></tr>\n");

	fprintf(html_fd, "<tr><td style=\"text-align: right\">This month's total: %s<br>" INTEXT " %s - " OUTTEXT "%s</td></tr>\n</table>\n",
		sizestr(total.rx + total.tx), sizestr(total.rx), sizestr(total.tx));

	fprintf(html_fd, "<p><img src=\"iograph.gif\" alt=\"logo\" style=\"float:left; margin-right: 1em\">"PACKAGE" - "VERSION"<br><a href=\"http://mdolze.gmxhome.de/\">mdolze.gmxhome.de</a></p>\n");
	fprintf(html_fd, "</body>\n</html>");
	fclose(html_fd);
}


/*
 * Creates the HTML file with the statistics for the past twelve months.
 */
void
monthlyhtml()
{
	FILE           *html_fd;
	int		a;
	unsigned long long bestmonth;
	float		hscale;
	bytecount	total;
	char		pname     [MAXPATHLEN];

	snprintf(pname, MAXPATHLEN, "%s/months.html", outdir);
	if ((html_fd = fopen(pname, "w")) == NULL)
		err(EX_CANTCREAT, "Cannot create months html file");

	/*
	 * Initializing to 1 does not distort results but prevents division
	 * by zero
	 */
	bestmonth = 1;
	for (a = 0; a <= 11; a++) {
		if (database->months[a].rx > bestmonth)
			bestmonth = database->months[a].rx;
		if (database->months[a].tx > bestmonth)
			bestmonth = database->months[a].tx;
	}
	hscale = (float)bestmonth / 450;

	fprintf(html_fd, "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n\"http://www.w3.org/TR/html4/loose.dtd\">\n");
	fprintf(html_fd, "<html>\n<head><title>"PACKAGE" - "VERSION"</title><meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\"></head>\n");
	fprintf(html_fd, "<body bgcolor=\"#FFFFFF\">\n");

	snprintf(pname, MAXPATHLEN, "%s/hourly.html", outdir);
	if (doesfileexist(pname))
		fprintf(html_fd, "<p><a href=\"hourly.html\">Today</a> | ");
	else
		fprintf(html_fd, "<p>Today | ");
	snprintf(pname, MAXPATHLEN, "%s/daily.html", outdir);
	if (doesfileexist(pname))
		fprintf(html_fd, "<a href=\"daily.html\">This month</a> | ");
	else
		fprintf(html_fd, "This month | ");
	fprintf(html_fd, "Previous Months</p>\n");

	fprintf(html_fd, "<h1 style=\"font-size:1.3em\">Monthly Network I/O for %s</h1>\n", hostname);
	fprintf(html_fd, "<table><tr><td><table cellpadding=\"5\" border=\"1\">\n");
	total.rx = total.tx = 0;

	for (a = 0; a <= 11; a++) {
		total.rx += database->months[a].rx;
		total.tx += database->months[a].tx;

		snprintf(pname, MAXPATHLEN, "%s/month%i.html", outdir, a + 1);
		/*
		 * Always link to the file if it exists. This creates a 'sliding window'
		 * over the past twelve months.
		 */
		if (doesfileexist(pname))
			fprintf(html_fd, "<tr><td><a href=\"month%i.html\"><b>%s</b></a>", a + 1,
				monthnames[a]);
		else
			fprintf(html_fd, "<tr><td><b>%s</b>", monthnames[a]);
		if (database->months[a].rx + database->months[a].tx > 0)
			fprintf(html_fd, "<br>%s</td>", sizestr(database->months[a].rx + database->months[a].tx));
		else
			fprintf(html_fd, "</td>");

		fprintf(html_fd, "<td width=\"455\"><img src=\"in.gif\" height=\"12\" width=\"%.f\" alt=\"ingraph\"><br>",
			database->months[a].rx / hscale);
		fprintf(html_fd, "<img src=\"out.gif\" height=\"12\" width=\"%.f\" alt=\"outgraph\"></td>",
			database->months[a].tx / hscale);
		fprintf(html_fd, "<td>" INTEXT " %s<br>" OUTTEXT " %s</td></tr>\n", sizestr(database->months[a].rx),
			sizestr(database->months[a].tx));
	}
	fprintf(html_fd, "</table></td></tr>\n</table>\n");
	fprintf(html_fd, "<p><img src=\"iograph.gif\" alt=\"logo\" style=\"float:left; margin-right: 1em\">"PACKAGE" - "VERSION"<br><a href=\"http://mdolze.gmxhome.de/\">mdolze.gmxhome.de</a></p>\n");
	fprintf(html_fd, "</body>\n</html>");
	fclose(html_fd);
}


/*
 * Read interface statistics from system and store in the struct passed as a
 * pointer. If there are no errors, it returns 0. If errors, returns -1.
 */
int
get_iface_stats(bytecount * bc, time_t * lastIfChange)
{
	int		rows;
	int		name       [6] = {CTL_NET, PF_LINK, NETLINK_GENERIC, IFMIB_IFDATA, 0, IFDATA_GENERAL};
	size_t		len;
	struct ifmibdata ifmd;	/* ifmibdata contains the network statistics */
	struct timeval	boottime;

	size_t		size = sizeof(boottime);
	if (sysctlbyname("kern.boottime", &boottime, &size, NULL, 0) < 0) {
		warn("sysctl kern.boottime failed");
		return -1;
	}
	printf("Machine bootet at: %s", ctime((time_t *) & boottime.tv_sec));

	printf("Getting interface stats for %s\n", ifname);

	len = sizeof(rows);
	/* get number of interfaces */
	if (sysctlbyname("net.link.generic.system.ifcount", &rows, &len, NULL, 0) == 0) {
		len = sizeof(ifmd);
		/*
		 * walk through all interfaces in the ifmib table from last
		 * to first
		 */
		for (; rows > 0; rows--) {
			name[4] = rows;	/* set the interface index */
			/* retrive the ifmibdata for the current index */
			if (sysctl(name, 6, &ifmd, &len, NULL, 0) == -1) {
				warn("read sysctl");
				break;
			}
			/* check if its interface name matches */
			if (strcmp(ifmd.ifmd_name, ifname) == 0) {
				bc->rx = ifmd.ifmd_data.ifi_ibytes;
				bc->tx = ifmd.ifmd_data.ifi_obytes;

				*lastIfChange = ifmd.ifmd_data.ifi_epoch + (time_t) boottime.tv_sec;

				printf("Last interface change: %s", ctime(lastIfChange));

				return 0;
			}
		}
		/* if we are here there is no interface with the given name */
		return -1;
	} else {
		warn("read sysctlbyname");
		return -1;
	}
}				/* get_iface_stats() */


/*
 * Checks if the given file exists. Return 1 if the file exists or 0
 * otherwise.
 */
int
doesfileexist(const char *f)
{
	int		fd;

	if (f == NULL)
		return 0;

	if ((fd = open(f, O_RDONLY)) != -1) {
		close(fd);
		return 1;
	}
	return 0;
}


/*
 * Returns the number of bytes given as argument NUM into a string with a
 * more readable KB, MB or GB representation.
 */
char           *
sizestr(unsigned long long num)
{
#define SIZESTR_SIZE 10
	char           *str = malloc(SIZESTR_SIZE);
	char           *format;
	float		val;

	if (num >= GBSIZE) {
		val = (((float)num) / GBSIZE);
		format = "%.1f GB";
	} else if (num >= MBSIZE) {
		val = (((float)num) / MBSIZE);
		format = "%.1f MB";
	} else if (num >= KBSIZE) {
		val = (((float)num) / KBSIZE);
		format = "%.1f KB";
	} else if (num >= 1) {
		val = (float)num;
		format = "%.f bytes";
	} else {
		snprintf(str, SIZESTR_SIZE, "0");
		return str;
	}

	snprintf(str, SIZESTR_SIZE, format, val);
	return str;
}


/*
 * Returns a textual representation for the given number of the day.
 */
char           *
daystr(int nday)
{
	static char	buf [5];
	switch (nday) {
	case 1:
		return "1st";
		/* NOTREACHED */
	case 2:
		return "2nd";
		/* NOTREACHED */
	case 3:
		return "3rd";
		/* NOTREACHED */
	case 21:
		return "21st";
		/* NOTREACHED */
	case 22:
		return "22nd";
		/* NOTREACHED */
	case 23:
		return "23rd";
		/* NOTREACHED */
	case 31:
		return "31st";
		/* NOTREACHED */
	default:
		snprintf(buf, sizeof(buf), "%ith", nday);
		return buf;
	}
}
